<?php

/**
 * @file
 * Implementation of the Flash question type.
 *
 * This question type only handles storing of data for a flash movie or any
 * other client side technology like a Java applet. All question logic is done
 * on the client side.
 *
 * The page containing this question will respond to the following get/post
 * variables:
 *
 * Get and Post:
 * - action=getAnswer: returns the current answer, or an empty page when there
 *   is no answer.
 * - action=getData: returns the current data field, or an empty page when there
 *   is no data.
 * - action=getInfo: return the current user's data, formatting in the way
 *   specified in the format field. The data consists of the following:
 *   - username: The current user's username.
 *   - firstname: If there is a profile field "profile_firstname" then the data
 *     of this field is returned.
 *   - middlename: If there is a profile field "profile_middlename" then the
 *     data of this field is returned.
 *   - lastname: If there is a profile field "profile_lastname" then the data
 *     of this field is returned.
 *   - tries: (int) The number in the tries field.
 *   - onceCorrect: (int) The number of tries that was listed the first time the
 *     correct variable was nonzero.
 * - format: The format to return the data of getInfo in. Possible values:
 *   - flash: return as flashvars (& seprated key=value pairs)
 *   - xml: xml data formatted as: <data><key1>value1</key1>...</data>
 *   - json: {"key":"value"}
 *
 * Post only:
 * - action=store: Will read (and store) the following variables, if set.
 *   - answer
 *   - data
 *   - tries
 *   - correct
 *
 */
class CqQuestionFlash extends CqQuestionAbstract {

  /**
   * HTML containing the flash tag and any accompanying text.
   *
   * @var string
   */
  private $text;

  /**
   * An optional database to perform queries against.
   *
   * @var array
   *   An associative array
   *   - server: string The database server name.
   *   - username: string The username to use to connect to the databse.
   *   - password: string The password to use to connect to the databse.
   *   - database: string The database to connect to.
   *   - queries: array with queries to execute. Each query is an associative
   *     array:
   *     - querystring: string The query to execute.
   *     - parameters: array with parameters to use in the query. Each parameter
   *       is an associative array:
   *       - type: string The type of parameter: remote, uid, uname.
   *       - name: string The name of the postvar to use.
   */
  private $database;

  /**
   * Constructs an option question object.
   * If there are data requests, we deal with them directly, except for database
   * requests, as we need the XML for those.
   *
   * @param CqUserAnswerInterface $userAnswer
   *   The CqUserAnswerInterface to use for storing the student's answer.
   * @param object $node
   *   Drupal node object that this question belongs to.
   */
  public function __construct(CqUserAnswerInterface &$userAnswer, &$node) {
    parent::__construct();
    $this->userAnswer = &$userAnswer;
    $this->node = &$node;

    if (isset($_REQUEST['action'])) {
      if (strtolower($_REQUEST['action']) == 'getanswer') {
        echo $this->userAnswer->getAnswer();
        die();
      }

      if (strtolower($_REQUEST['action']) == 'getdata') {
        echo $this->userAnswer->getData('data');
        die();
      }

      if (strtolower($_REQUEST['action']) == 'getinfo') {
        global $user;
        profile_load_profile(&$user);

        $info = array();
        $info['username'] = $user->name;
        if (isset($user->profile_firstname)) {
          $info['firstname'] = $user->profile_firstname;
        }
        if (isset($user->profile_middlename)) {
          $info['middlename'] = $user->profile_middlename;
        }
        if (isset($user->profile_lastname)) {
          $info['lastname'] = $user->profile_lastname;
        }
        $info['tries'] = $this->userAnswer->getTries();
        $info['onceCorrect'] = $this->userAnswer->onceCorrect();

        $format = '';
        if (isset($_REQUEST['format'])) {
          $format = strtolower($_REQUEST['format']);
        }
        switch ($format) {
          case 'xml':
            $dom = new DOMDocument('1.0', 'utf-8');
            $root = $dom->createElement('data');
            $dom->appendChild($root);
            foreach ($info AS $key => $value) {
              $node = $dom->createElement($key, $value);
              $root->appendChild($node);
            }
            echo $dom->saveXML();
            die;
            break;

          case 'json':
            drupal_json($info);
            die();
            break;

          case 'flash':
          default:
            foreach ($info AS $key => $value) {
              echo '&' . $key . '=' . urlencode($value) . '&';
            }
            die;
            break;
        }
      }
    }

    if (isset($_POST['action'])) {
      if (strtolower($_POST['action']) == 'store') {
        $dataSize = 0;
        if (isset($_POST['answer'])) {
          $this->userAnswer->setAnswer($_POST['answer']);
          $dataSize += strlen($_POST['answer']);
        }
        if (isset($_POST['data'])) {
          $this->userAnswer->setData('data', $_POST['data']);
          $dataSize += strlen($_POST['data']);
        }
        if (isset($_POST['tries'])) {
          $this->userAnswer->setTries($_POST['tries']);
        }
        if (isset($_POST['correct'])) {
          $this->userAnswer->setCorrect($_POST['correct']);
          $this->userAnswer->setData('correct', $_POST['correct']);
        }
        $this->userAnswer->store();
        $format = '';
        if (isset($_REQUEST['format'])) {
          $format = strtolower($_REQUEST['format']);
        }
        switch ($format) {
          case 'xml':
            $dom = new DOMDocument('1.0', 'utf-8');
            $root = $dom->createElement('result');
            $dom->appendChild($root);
            $node = $dom->createElement('success', 'true');
            $root->appendChild($node);
            $node = $dom->createElement('size', $dataSize);
            $root->appendChild($node);
            echo $dom->saveXML();
            die;
            break;

          case 'json':
            drupal_json(array('success' => TRUE, 'size' => $dataSize));
            die();
            break;

          case 'flash':
          default:
            echo '&success=true&size=' . $dataSize . '&';
            die;
            break;
        }
      }
    }
  }

  /**
   * Implements CqQuestionAbstract::getOutput()
   */
  public function getOutput() {
    $this->initialise();
    $retval = $this->prefix;
    $retval .= drupal_get_form('closedquestion_get_form_for', $this->node);
    $retval .= $this->postfix;
    return $retval;
  }

  /**
   * Implements CqQuestionAbstract::getFeedbackItems()
   */
  public function getFeedbackItems() {
    $feedback = array();
    return $feedback;
  }

  /**
   * Overrides CqQuestionAbstract::loadXml()
   * If there is a database request we deal with it now.
   */
  public function loadXml(DOMElement $dom) {
    global $user;
    parent::loadXml($dom);
    module_load_include('inc.php', 'closedquestion', 'lib/XmlLib');

    foreach ($dom->childNodes as $node) {
      $name = drupal_strtolower($node->nodeName);
      switch ($name) {
        case 'text':
          $this->text = cq_get_text_content($node, $this);
          break;

        case 'database':
          $dbdata = array();
          $dbdata['queries'] = array();

          $attribs = $node->attributes;
          $item = $attribs->getNamedItem('server');
          if ($item !== NULL) {
            $dbdata['server'] = $item->value;
          }
          $item = $attribs->getNamedItem('username');
          if ($item !== NULL) {
            $dbdata['username'] = $item->value;
          }
          $item = $attribs->getNamedItem('password');
          if ($item !== NULL) {
            $dbdata['password'] = $item->value;
          }
          $item = $attribs->getNamedItem('database');
          if ($item !== NULL) {
            $dbdata['database'] = $item->value;
          }

          foreach ($node->childNodes as $querychild) {
            switch ($querychild->nodeName) {
              case 'query':
                $querydata = array();
                $attribs = $querychild->attributes;
                $item = $attribs->getNamedItem('string');
                if ($item !== NULL) {
                  $querydata['string'] = $item->value;
                }

                foreach ($querychild->childNodes as $paramchild) {
                  switch ($paramchild->nodeName) {
                    case 'param':
                      $paramdata = array();
                      $attribs = $paramchild->attributes;
                      $item = $attribs->getNamedItem('type');
                      if ($item !== NULL) {
                        $paramdata['type'] = $item->value;
                      }
                      $item = $attribs->getNamedItem('name');
                      if ($item !== NULL) {
                        $paramdata['name'] = $item->value;
                      }
                      if ($paramdata['type']) {
                        $querydata['parameters'][] = $paramdata;
                      }
                  }
                }

                if ($querydata['string']) {
                  $dbdata['queries'][] = $querydata;
                }
            }
          }

          if ($dbdata['server'] && $dbdata['database']) {
            $this->database = $dbdata;
          }
          break;

        default:
          if (!in_array($name, $this->knownElements)) {
            drupal_set_message(t('Unknown node: @nodename', array('@nodename' => $node->nodeName)));
          }
          break;
      }
    }

    // We now have the database information. Lets see if we need to do a query.
    // In that case we can stop loading and exit.
    if (isset($_POST['action'])) {
      if (strtolower($_POST['action']) == 'query') {
        if (isset($_POST['query'])) {
          $queryId = (int) $_POST['query'];
          if (isset($this->database['queries'][$queryId])) {
            $querydata = $this->database['queries'][$queryId];
            $params = array();
            if ($querydata['parameters']) {
              foreach ($querydata['parameters'] AS $param) {
                switch ($param['type']) {
                  case 'uid':
                    $params[] = $user->uid;
                    break;

                  case 'uname':
                    $params[] = $user->name;
                    break;

                  case 'remote':
                    if (isset($_POST[$param['name']])) {
                      $params[] = $_POST[$param['name']];
                    }
                    break;
                }
              }
            }

            // To connect to the database we need to do some juggling.
            // see http://api.drupal.org/api/drupal/includes--database.inc/function/db_set_active/6
            $dbName = 'closedQuestionFlashTemp';
            global $db_url;
            if (is_string($db_url)) {
              $db_url = array('default' => $db_url);
            }
            $db_url[$dbName] = substr($db_url['default'], 0, strpos($db_url['default'], '://'))
              . '://' . $this->database['username']
              . ':' . $this->database['password']
              . '@' . $this->database['server']
              . '/' . $this->database['database'];
            $currentDb = db_set_active($dbName);
            $result = db_query($querydata['string'], $params);
            $data = array();
            if (!is_bool($result)) {
              while ($row = db_fetch_array($result)) {
                $data[] = $row;
              }
            }
            if ($currentDb) {
              db_set_active($currentDb);
            }

            $format = '';
            if (isset($_REQUEST['format'])) {
              $format = strtolower($_REQUEST['format']);
            }
            switch ($format) {
              case 'xml':
                $dom = new DOMDocument('1.0', 'utf-8');
                $root = $dom->createElement('data');
                $dom->appendChild($root);
                foreach ($data AS $row) {
                  $rownode = $dom->createElement('row');
                  foreach ($row AS $key => $value) {
                    $node = $dom->createElement($key, $value);
                    $rownode->appendChild($node);
                  }
                  $root->appendChild($rownode);
                }
                echo $dom->saveXML();
                die();
                break;

              case 'json':
              default;
                drupal_json($data);
                die();
                break;
            }
          }
        }
      }
    }
  }

  /**
   * Implements CqQuestionAbstract::getForm()
   */
  public function getForm($formState) {

    $form['questionText'] = array(
      '#type' => 'item',
      '#value' => $this->text,
    );

    return $form;
  }

  /**
   * Implements CqQuestionAbstract::checkCorrect()
   */
  public function checkCorrect() {
    dpm('checkCorrect()');
    return 0;
  }

  /**
   * Implements CqQuestionAbstract::submitAnswer()
   */
  public function submitAnswer($form, &$form_state) {
    dpm('submitAnswer()');
    dpm($_REQUEST);
  }

  /**
   * Implements CqQuestionAbstract::getAllText()
   */
  public function getAllText() {
    $this->initialise();
    $retval = '';
    $retval .= closedquestion_make_fieldset(t('Text'), $this->text);
    return $retval;
  }

}
